# -*- coding: utf-8 -*-

from time import localtime, strftime, mktime
from datetime import datetime
from collections import OrderedDict

from django.core.cache import cache
from django.urls import reverse
from django.http import HttpResponseServerError, Http404
from django.shortcuts import render
from django.template.loader import render_to_string

from ldap_inp import LdapEleve, L
from lib.access_auth import access_restricted, none_if_anonymous
from portailuser.models import Ecole


@none_if_anonymous
def widget(request):
    nb_annivs = 0
    annivs = cache.get("annivs")
    annivs = None

    if annivs is None:
        annivs = []
        year = int(strftime("%Y", localtime()))
        if int(strftime("%m", localtime())) > 8:
            max_promo = year + 3
        else:
            max_promo = year + 2

        day = localtime(mktime(localtime()))

        if request.ecole == Ecole.inp:
            ldap_annivs = LdapEleve.objects.filter(L(birthdate__matching=strftime("*-%m-%d", day)))
        else:
            ldap_annivs = LdapEleve.objects.filter(
                L(ecole=request.ecole) & L(birthdate__matching=strftime("*-%m-%d", day))
            )

        for ldap_anniv in ldap_annivs:
            if ldap_anniv.promo > max_promo - 3:
                anniv = {}
                anniv["nom"] = ldap_anniv.sn
                anniv["prenom"] = ldap_anniv.givenName
                anniv["age"] = year - int(ldap_anniv.birthdate.year)
                anniv["lien"] = reverse("annuaire:fiche", args=[ldap_anniv.uid])
                nb_annivs += 1
                annivs.append(anniv)
        if nb_annivs == 0:
            annivs = None
        cache.set("annivs", annivs, 3600 * 4)

    c = {
        "annivs": annivs,
    }

    return render_to_string("anniversaire/widget.html", c)


@access_restricted
def liste(request):
    year = int(strftime("%Y", localtime()))

    if int(strftime("%m", localtime())) > 8:
        max_promo = year + 3
    else:
        max_promo = year + 2

    try:
        eleve = request.user.get_ldap_user()
    except LdapEleve.DoesNotExist:
        raise Http404

    mes_groupes = set()
    for groupe in eleve.get_groupes():
        mes_groupes.add(groupe.cn)

    annivs = []
    filieres = {}
    for i in range(7):
        danniv = OrderedDict()  # anniversaires du jour
        day = localtime(mktime(localtime()) + 3600 * 24 * i)
        if request.ecole == Ecole.inp:
            ldap_annivs = LdapEleve.objects.filter(L(birthdate__matching=strftime("*-%m-%d", day)))
        else:
            ldap_annivs = LdapEleve.objects.filter(
                L(ecole=request.ecole) & L(birthdate__matching=strftime("*-%m-%d", day))
            )
        danniv[max_promo - 2] = {}
        danniv[max_promo - 1] = {}
        danniv[max_promo] = {}
        for ldap_anniv in ldap_annivs:
            if max_promo >= ldap_anniv.promo > max_promo - 3:
                filiere = ldap_anniv.filiere.get()
                filieres.update({filiere.ou: {"acronyme": filiere.ou, "nom_court": filiere.shortName}})
                ses_groupes = set_of_gp(ldap_anniv.uid)
                anniv = {
                    "nom": ldap_anniv.sn,
                    "prenom": ldap_anniv.givenName,
                    "age": int(strftime("%Y", day)) - int(ldap_anniv.birthdate.year),
                    "lien": reverse("annuaire:fiche", args=[ldap_anniv.uid]),
                    "filiere": filiere.ou,
                }
                if len(mes_groupes & ses_groupes) > 0 or (
                    ldap_anniv.promo == eleve.promo and ldap_anniv.filiere.get() == eleve.filiere.get()
                ):
                    anniv["connu"] = True
                else:
                    anniv["connu"] = False
                anniv_promo = danniv[ldap_anniv.promo]
                if ldap_anniv.filiere.get().shortName not in anniv_promo:
                    anniv_promo[ldap_anniv.filiere.get().shortName] = []
                anniv_promo[ldap_anniv.filiere.get().shortName].append(anniv)
        annivs.append((danniv, datetime.fromtimestamp(mktime(localtime()) + 3600 * 24 * i)))

    c = {
        "annivs": annivs,
        "filieres": filieres,
    }

    return render(request, "anniversaire/liste.html", c)


def set_of_gp(uid):
    s = set()
    try:
        eleve = LdapEleve.objects.get(uid=uid)
    except LdapEleve.DoesNotExist:
        return HttpResponseServerError("Erreur de récupération des informations")
    for groupe in eleve.get_groupes():
        s.add(groupe.cn)
    return s
